﻿# ------------------------------------------------------------------------------
#
# FishFaTX.py                                            Ulrich Mueller 18.01.10
#                                                        PCS1700    Python 3.1.1
# Modul FishFaTX zum Betrieb des fischertechnik ROBO TX Controllers
# Klasse FishFace : Kapselung und Erweiterung der Funktionen von umFish50.DLL
#                   benoetigt umFish50.DLL und ftMscLib.DLL
# Klasse FishFaceException : Raise FishFace Ausnahmen
#
# ------------------------------------------------------------------------------

from ctypes import *

class FishFaceException(IOError):
  'Allgemeiner FishFace Fehler'

class FishFace(object):

  iOpen        = -1          
                             # gleichzeitig Open-Merker (no Open = -1)
  PollInterval = 12
  
# ---- symbolische Konstanten -------------------
  Fehler       = 0xE0000001
  Ein          = 1
  Aus          = 0
  Links        = 1
  Rechts       = 2
  
# --- interne Werte ---------------------------
  ESCAPE       = 27
  
  def __init__(self):
    self.iOpen = -1
    
# --- oeffentliche Methoden und Eigenschaften ---

  def OpenController(self, ComNr):
    "Verbindung zum Controller herstellen"
    self.iOpen = windll.umfish50.txOpenController(ComNr)
    print ("OpenController : COM", ComNr)
    if self.iOpen == self.Fehler:
      raise (FishFaceException, 'ControllerProblem.Open')  

  def ClearCounter(self, ctrID, InputNr):
    "Loeschen eines Counters"
    windll.umFish50.txClearCounter(ctrID, InputNr-1)

  def CloseController(self):
    "Schliessen Controller"
    windll.umFish50.txCloseController()
    self.iOpen = -1

  def Finish(self, ctrID=0, InputNr=0):
    "Feststellen Endewunsch (Escape, InputNr)"
    if windll.user32.GetAsyncKeyState(self.ESCAPE) != 0: return 1
    if InputNr > 0 and self.GetInput(ctrID, InputNr): return 1
    return 0
    
  def GetAnalog(self, ctrID, AnalogNr):
    "Auslesen Analogwert eines A5K-Einganges"
    return windll.umFish50.txGetAnalog(ctrID, AnalogNr-1)
  
  def GetCounter(self, ctrID, InputNr):
    "Auslesen eines Counters"
    return windll.umFish50.txGetCounter(ctrID, InputNr-1)

  def GetDistance(self, ctrID, InputNr):
    "Auslesen des Abstandswertes"
    return windll.umFish50.txGetDistance(ctrID, InputNr-1)
  
  def GetInput(self, ctrID, InputNr):
    "Bestimmen Status I-Eingang"
    return windll.umFish50.txGetInput(ctrID, InputNr-1)
  
  def GetTrack(self, ctrID, InputNr):
    "Auslesen des Spurwertes"
    return windll.umFish50.txGetTrack(ctrID, InputNr-1);
  
  def GetVoltage(self, ctrID, VoltNr):
    "Auslesen Voltwert eines A10V-Einganges"
    return windll.umFish50.txGetVoltage(ctrID, VoltNr-1)
  
  def Pause(self, mSek):
    "Programm anhalten"    
    sZeit = windll.kernel32.GetTickCount()
    while (windll.kernel32.GetTickCount() - sZeit) < mSek:
      if windll.user32.GetAsyncKeyState(self.ESCAPE) != 0 : return
      windll.kernel32.Sleep(self.PollInterval)
    
  def SetLamp(self, ctrID, LampNr, OnOff):
    "Setzen eines O-Ausganges"
    windll.umFish50.txSetLamp(ctrID, LampNr-1, OnOff)
    
  def SetMotor(self, ctrID, MotorNr, Direction, Speed=512):
    "Setzen eines M-Ausganges"      
    windll.umFish50.txSetMotor(ctrID, MotorNr-1, Direction, Speed)

  def StartMotor(self, ctrID, MotorNr, Direction, Speed, ICount):
    "Starten eines Motors für ICount Impulse"
    windll.umFish50.txStartMotor(ctrID, MotorNr-1, Direction, Speed, ICount, 0)

  def StartRobMotor(self, ctrID, MotorNr, Direction, Speed, ICount):
     "Starten eines RobMotors für ICount Impulse"
     windll.umFish50.txStartMotor(ctrID, MotorNr-1, Direction, Speed, ICount, 1)
  
  def Version(self):
    "Version der Klasse"
    return "5.0.01"

  def WaitForHigh(self, ctrID, InputNr):
    "Warten auf einen false/true-Durchgang von InputNr"
    self.WaitForInput(ctrID, InputNr, 0)
    self.WaitForInput(ctrID, InputNr, 1)

  def WaitForInput(self, ctrID, InputNr, OnOff=1):
    "Warten auf InputNr = OnOff"
    while not (self.GetInput(ctrID, InputNr) == OnOff):
      windll.kernel32.Sleep(self.PollInterval)
      if windll.user32.GetAsyncKeyState(self.ESCAPE) != 0 : return

  def WaitForLow(self, ctrID, InputNr):
    "Warten auf einen true/false-Durchgang von InputNr"
    self.WaitForInput(ctrID, InputNr, 1)
    self.WaitForInput(ctrID, InputNr, 0)

  def WaitForMotor(self, ctrID, MotorNr):
    "Warten auf das MotorReady-Ereignis"
    res = -1
    while res < 0 and windll.user32.GetAsyncKeyState(self.ESCAPE) == 0:
      res = windll.umFish50.txIsMotorReady(ctrID, MotorNr-1)
    return res

  def WaitForMotors(self, ctrID, MotorNrs):
    "Warten auf MotorReady-Ereignis aller Motoren"
    c = 1
    while c != 0 and windll.user32.GetAsyncKeyState(self.ESCAPE) == 0:
      c = 0
      for i in MotorNrs:
        res = windll.umFish50.txIsMotorReady(ctrID, i-1)
        if res < 0: c += 1
